> -- |The Data.Lattice module defines the class of lattices,
> -- i.e. types with meet ('/\') and join ('\/') operations satisfying the
> -- properties of boolean lattices.

> module Data.Lattice where

> -- |The 'Lattice' class defines boolean-like lattice structures with
> -- '/\' and '\/' (meet and join) operations. Minimal and maximal
> -- elements are specified as Maybes.
> class Lattice a where
>     lempty :: Maybe a
>     lfull :: Maybe a
>     (/\) :: a -> a -> a
>     (\/) :: a -> a -> a 
>     meet :: [a] -> Maybe a 
>     meet l | null l    = lempty 
>            | otherwise = Just (foldr1 (\/) l)
>     join :: [a] -> Maybe a 
>     join l | null l    = lfull
>            | otherwise = Just (foldr1 (/\) l)

Booleans have the classical lattice structure using standard Haskell
functions.

> instance Lattice Bool where
>     lempty = Just False
>     lfull = Just True
>     (/\) = (&&)
>     (\/) = (||)
>     meet = Just . and
>     join = Just . or

 instance (Integral a) => Lattice a where
     lempty = Just 1
     lfull = Nothing
     (/\) = gcd
     (\/) = lcm

