> module Data.MonoidPlus where 

We will use the Monoid module for basic classes. 
 
> import Data.Monoid


> -- |A class for ordered monoids.
> class (Monoid a, Ord a) => OrdMonoid a

> {- |
>    The class 'GoodMonoid' of good monoids is the preferred class to use to
>    generate rings with good properties, for example rings with a Show
>    method. 
>  -}
> class (OrdMonoid a, Show a) => GoodMonoid a

> -- |Define a class for graded monoids, like free commutative monoids. 
> class (Monoid a) => GradedMonoid a where 
>     degree :: a -> Integer

> -- |The 'GroebnerMonoid' class gathers monoids with enough structure
> -- to allow computation of Groebner bases
> class (GoodMonoid a) => GroebnerMonoid a where
>     -- |Multiplication
>     (×) :: a -> a -> a
>     -- |Division, which should only be used if exponents remain positive
>     (÷) :: a -> a -> a
>     -- |Returns the least common multiple of monomials.
>     lcm_monom :: a -> a -> a
>     -- |Returns the least common multiple of monomials and the
>     -- complements. 
>     lcm2_monom :: a -> a -> (a,a)
>     -- |Returns the greatest common divisor of monomials.
>     gcd_monom :: a -> a -> a
>     -- |Tests divisibility
>     divides :: a -> a -> Bool

