> module Data.Polynomial where 
> import qualified Data.Map as Map
> import Data.Array.IArray
> import Data.Monomial
> import Data.MonoidRing
> import Data.Ratio

We will use the Monomial module to define free semi-groups, and the
   MonoidAlgebra module to define the algebra of a free semi-group,
   which is no less than the free polynomial algebra.

> type Polynom r m = MonoidRing r (Monom m)

The functor (Polynom r) has a monad structure, but requires Ord m 
    and Eq m. However there should be monad-like transformations

> returnp :: (Num r, VarList m) => m -> Polynom r m
> returnp x = MakePolynom (Map.singleton xm 1) where
>         xm = returnm x

We define several well-known sets of variables.

> data VarXYZTUVW = X | Y | Z | T | U | V | W deriving (Enum, Eq, Ord, Bounded)
> instance Show VarXYZTUVW where show v = ["xyztuvw"!!(fromEnum v)]
> instance Ix VarXYZTUVW where 
>     range = uncurry enumFromTo 
>     index (x,y) z = fromEnum z - fromEnum x
>     inRange (x,y) z = (x <= z) && (z <= y)
> instance VarList VarXYZTUVW

> type VarXYZ = VarXYZTUVW
> type VarXYZT = VarXYZTUVW
> type VarXYZTUV = VarXYZTUVW
> type VarXYZUVW = VarXYZTUVW

> data VarXInt = VarX Integer deriving (Eq, Ord)
> instance Show VarXInt where show (VarX v) = 'X':show v

A human-readable field of rational numbers.

> data QQ = ToQQ Rational deriving (Eq, Ord)
> instance Num QQ where
>     (ToQQ x) + (ToQQ y) = ToQQ (x+y)
>     (ToQQ x) - (ToQQ y) = ToQQ (x-y)
>     (ToQQ x) * (ToQQ y) = ToQQ (x*y)
>     abs (ToQQ x) = ToQQ (abs x)
>     signum (ToQQ x) = ToQQ (signum x)
>     fromInteger n = ToQQ (fromInteger n)
> instance Fractional QQ where
>     (ToQQ x) / (ToQQ y) = ToQQ (x/y)
>     fromRational n = ToQQ (fromRational n)
> instance Show QQ where
>     show (ToQQ x) | q == 1    = show p
>                   | otherwise = shows p ('/':show q)
>         where p = numerator x
>               q = denominator x
