> module Data.Ring where

> import Data.List
> import Data.Lattice

> -- |This defines a 'Ring' Class with 'zero', sum '+°' and product
> -- |'*°'. Rign operations are represented by a small ring "°". 
> class (Eq a) => Ring a where 
>     zero :: a
>     zero = fromInt 0
>     fromInt :: Integer -> a 
>     (+°) :: a -> a -> a 
>     (-°) :: a -> a -> a 
>     (*°) :: a -> a -> a 
>     minus :: a -> a
>     a -° b = a +° (minus b)

> -- |Ideals of a ring are represented by lists of elements. 
> data (Ring a) => Ideal a = MakeIdeal [a]
> fromIdeal (MakeIdeal i) = i
> toIdeal i = MakeIdeal i

> principal_ideal :: (Ring a) => a -> Ideal a
> -- |The 'principal_ideal' constructor creates a principal ideal from
> -- an element of the ring
> principal_ideal x = MakeIdeal [x]

> add_ideals :: (Ring a) => Ideal a -> Ideal a -> Ideal a 
> -- |Ideals can be added.
> add_ideals (MakeIdeal i) (MakeIdeal j) = MakeIdeal (i++j)

> mult_ideals :: (Ring a) => Ideal a -> Ideal a -> Ideal a 
> -- |Ideals can be multiplied.
> mult_ideals (MakeIdeal i) (MakeIdeal j) = 
>     MakeIdeal [a *° b | a <- i, b <- j]

> instance (Ring a, Show a) => Show (Ideal a)

> instance (Ring a) => Lattice (Ideal a) where
>     lempty = Just (MakeIdeal [])
>     lfull = Just (MakeIdeal [fromInt 1])
>     (/\) = mult_ideals
>     (\/) = add_ideals 